#!/usr/bin/env python3
# -*- coding: utf-8 -*

import gi
gi.require_version('Gtk', '3.0')
from gi.repository import Gtk,GObject, GLib,Gio,Gdk

import threading
import copy
import time
import sys
import socket
import os

import signal
signal.signal(signal.SIGINT, signal.SIG_DFL)
import N4dManager
import PanelDesignManager

import gettext
gettext.textdomain('panel-design-manager')
_ = gettext.gettext


class PanelDesignWindow:
	
	DEBUG=True
	
	def __init__(self):
		
		self.n4dman=N4dManager.N4dManager("server")
		self.pdm=PanelDesignManager.PanelDesignManager()

		self.new_conf={}
		self.new_conf["dconf"]=None
		self.new_conf["date"]=None
		self.new_conf["replicate"]=True
		self.new_conf["plank"]=True
		
		
		self.current_conf={}
		self.current_conf["dconf"]=None
		self.current_conf["date"]=None
		self.current_conf["replicate"]=True
		self.current_conf["plank"]=True
				
		self.start_gui()
		
	#def init

	
	def dprint(self,data):
		
		if PanelDesignWindow.DEBUG:
			print("[panel-design-manager] %s"%str(data))
		
	#def dprint 
	
	
	def where_am_i(self):
		
		server="localhost"
		
		try:
		
			if os.path.exists("/usr/bin/lliurex-version"):
				is_server=os.system("lliurex-version -t server")
				is_client=os.system("lliurex-version -t client")
				try:
					server_known=socket.gethostbyname("server")
				except:
					server_known=None
				
				if is_server==0 or is_client==0 or server_known!=None:
					server=""
					
			else:
				server="localhost"
			
		except Exception as e:
			
			self.dprint(e)
			server="localhost"
		
		return server
		
	#def where_am_i
	
	
	def start_gui(self):
		
		builder=Gtk.Builder()
		builder.set_translation_domain('panel-design-manager')
		builder.add_from_file("/usr/share/panel-design-manager/rsrc/panel_manager.ui")
		
		self.main_window=builder.get_object("window")
		self.window_vbox=builder.get_object("window_vbox")
		self.main_box=builder.get_object("main_box")

		self.enable_design_label=builder.get_object("enable_design_label")
		self.panel_design_label=builder.get_object("panel_design_label")
		self.panel_info_label=builder.get_object("panel_info_label")
		self.show_plank_label=builder.get_object("show_plank_label")
		self.plank_info_label=builder.get_object("plank_info_label")
		self.configuration_label=builder.get_object("configuration_label")
		self.replication_label=builder.get_object("replication_label")
		self.msg_label=builder.get_object("msg_label")
		
		self.configuration_image=builder.get_object("configuration_image")
		
		self.copy_button=builder.get_object("copy_button")
		self.options_toolbutton=builder.get_object("options_toolbutton")
		self.apply_button=builder.get_object("apply_button")
		
		self.main_frame=builder.get_object("main_frame")
		
		self.enable_design_switch=builder.get_object("enable_design_switch")
		self.show_plank_switch=builder.get_object("show_plank_switch")
		self.replication_switch=builder.get_object("replicate_switch")
		
		self.login_box=builder.get_object("login_box")
		self.user_entry=builder.get_object("user_entry")
		self.password_entry=builder.get_object("password_entry")
		self.server_ip_entry=builder.get_object("server_ip_entry")
		self.login_button=builder.get_object("login_button")
		self.login_msg_label=builder.get_object("login_msg_label")
		
		self.changes_dialog=builder.get_object("changes_dialog")
		self.exit_button=builder.get_object("exit_button")
		self.cancel_button=builder.get_object("cancel_button")
		self.changes_label=builder.get_object("changes_label")
		
		self.stack=Gtk.Stack()
		self.stack.set_transition_type(Gtk.StackTransitionType.CROSSFADE)
		self.stack.set_transition_duration(200)
		self.stack.add_titled(self.login_box,"login","Login")
		self.stack.add_titled(self.main_box,"main","Main Box")
		self.stack.show_all()
		
		if self.where_am_i()=="localhost":
			self.server_ip_entry.set_text("localhost")
		
		self.options_popover=Gtk.Popover()
		self.options_popover.set_relative_to(self.options_toolbutton)
		
		v=Gtk.VBox()
		self.import_button=Gtk.Button("Import")
		v.pack_start(self.import_button,True,True,5)
		self.export_button=Gtk.Button("Export")
		v.pack_start(self.export_button,True,True,5)
		self.options_popover.add(v)
		
		self.window_vbox.pack_start(self.stack,True,True,0)
		
		self.set_css_info()
		self.connect_signals()
		
		self.login_button.grab_focus()
		self.apply_button.hide()
		self.main_window.show()
		GObject.threads_init()
		Gtk.main()
		
	#def start_gui


	def set_css_info(self):
		
		self.style_provider=Gtk.CssProvider()
		f=Gio.File.new_for_path("/usr/share/panel-design-manager/rsrc/style.css")
		self.style_provider.load_from_file(f)
		Gtk.StyleContext.add_provider_for_screen(Gdk.Screen.get_default(),self.style_provider,Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION)
		
		self.main_window.set_name("WHITE-BACKGROUND")
		self.enable_design_label.set_name("HEADER-LABEL")
		self.panel_design_label.set_name("DEFAULT-LABEL")
		self.show_plank_label.set_name("DEFAULT-LABEL")
		self.replication_label.set_name("DEFAULT-LABEL")
		
		self.changes_label.set_name("DEFAULT-LABEL")
		
		self.panel_info_label.set_name("INFO-LABEL")
		self.plank_info_label.set_name("INFO-LABEL")
		self.configuration_label.set_name("INFO-LABEL")
		
		self.import_button.set_name("BORDERLESS-BUTTON")
		self.export_button.set_name("BORDERLESS-BUTTON")
		
	#def set-css_info	


	def connect_signals(self):
		
		self.main_window.connect("destroy",Gtk.main_quit)
		self.main_window.connect("delete_event",self.check_changes)
		self.enable_design_switch.connect("notify::active",self.enable_design_switch_changed)
		
		self.options_toolbutton.connect("clicked",self.options_clicked)
		self.copy_button.connect("clicked",self.copy_clicked)
		self.apply_button.connect("clicked",self.apply_clicked)
		
		self.import_button.connect("clicked",self.import_clicked)
		self.export_button.connect("clicked",self.export_clicked)
		
		
		self.login_button.connect("clicked",self.login_clicked)
		self.user_entry.connect("activate",self.entries_press_event)
		self.password_entry.connect("activate",self.entries_press_event)
		self.server_ip_entry.connect("activate",self.entries_press_event)
		
		self.changes_dialog.connect("delete_event",self.dialog_closed)
		self.exit_button.connect("clicked",Gtk.main_quit)
		self.cancel_button.connect("clicked",self.dialog_closed,None)
		
	#def connect_signals


	def load_current_configuration(self):
		
		self.current_conf=self.n4dman.get_configuration()
		if self.current_conf!=None:
			self.main_frame.set_sensitive(self.current_conf["status"])
			self.enable_design_switch.set_active(self.current_conf["status"])
			self.show_plank_switch.set_active(self.current_conf["plank"])
			self.replication_switch.set_active(self.current_conf["replicate"])
			if self.current_conf["date"]!=None:
				self.configuration_label.set_text(_("Configuration date") +": "+self.current_conf["date"])
				self.configuration_image.set_from_stock(Gtk.STOCK_YES,Gtk.IconSize.BUTTON)
			
		
	#def load_current_configuration
	
	def check_changes(self,widget,event):
		
		if self.stack.get_visible_child_name()=="main":
			self.gather_values()
		
		if self.new_conf!=self.current_conf:
			self.changes_dialog.show()
			return True
		
		Gtk.main_quit()
		
	#def check_changes


	def gather_values(self):
	
		self.new_conf["status"]=self.enable_design_switch.get_active()
		self.new_conf["plank"]=self.show_plank_switch.get_active()
		self.new_conf["replicate"]=self.replication_switch.get_active()
		if self.new_conf["dconf"]==None:
			self.new_conf["dconf"]=self.current_conf["dconf"]
		if self.new_conf["date"]==None:
			self.new_conf["date"]=self.current_conf["date"]
		
	#def gather_values
	
	
	def enable_design_switch_changed(self,switch,data):
		
		self.main_frame.set_sensitive(switch.get_active())
		
	#def enable_design_switch_changed
	

	def dialog_closed(self,widget,event):
		
		self.changes_dialog.hide()
		return True
		
	#def dialog_closed

	
	def entries_press_event(self,widget):
		
		self.login_clicked(None)
		
	#def entries_press_event
	
	
	def copy_clicked(self,button):
		
		self.dprint("COPY")
		
		self.new_conf["dconf"]=self.pdm.get_dconf()
		self.new_conf["date"]=time.strftime("%d/%m/%Y")
		
		self.configuration_label.set_text(_("Configuration date")+": "+self.new_conf["date"])
		self.configuration_image.set_from_stock(Gtk.STOCK_YES,Gtk.IconSize.BUTTON)
		
	#def copy_clicked
	
	
	def apply_clicked(self,button):
		
		self.dprint("APPLY")
		self.gather_values()
		
		if self.new_conf["dconf"]!=None:
			self.dprint("SAVING...")
			ret=self.n4dman.save_configuration(self.new_conf)
			self.dprint(ret)
			self.current_conf=copy.deepcopy(self.new_conf)
			self.msg_label.set_markup("<span foreground='#0062f5'>"+_("Configuration saved")+"</span>")
			GLib.timeout_add(5000,self.remove_msg_txt)
		else:
			self.msg_label.set_markup("<span foreground='red'>"+_("There was an error when saving configuration.")+"</span>")
		
	#def apply_clicked
	
	
	def remove_msg_txt(self):
		
		self.msg_label.set_text("")
		
	#def remove_msg_txt
	
	
	def login_clicked(self,widget):
		
		user=self.user_entry.get_text()
		password=self.password_entry.get_text()
		server=self.server_ip_entry.get_text()

		if server!="":
			self.n4dman.set_server(server)
		#else server='server'
		
		if server=="localhost":
			self.replication_label.set_sensitive(False)
			self.replication_switch.set_sensitive(False)
		
		self.login_msg_label.set_text(_("Validating user..."))
		
		self.login_button.set_sensitive(False)
		self.validate_user(user,password)
		
	#def login_clicked
	
	
	def validate_user(self,user,password):
		
		t=threading.Thread(target=self.n4dman.validate_user,args=(user,password,))
		t.daemon=True
		t.start()
		GLib.timeout_add(500,self.validate_user_listener,t)
		
	#def validate_user
	
	
	def validate_user_listener(self,thread):
			
		if thread.is_alive():
			return True
				
		self.login_button.set_sensitive(True)
		if not self.n4dman.user_validated:
			self.login_msg_label.set_markup("<span foreground='red'>"+_("Invalid user")+"</span>")
		else:
			group_found=False
			for g in ["adm","admins"]:
				if g in self.n4dman.user_groups:
					group_found=True
					break
					
			if group_found:
				self.login_msg_label.set_text("")
				self.load_current_configuration()
				self.stack.set_visible_child_name("main")
				self.apply_button.show()
				
			else:
				self.login_msg_label.set_markup("<span foreground='red'>"+_("User must be in 'adm' or 'admins' group")+"</span>")
				
		return False
			
	#def validate_user_listener
	
	
	# ## POPOVER FUNCTIONS ## ########################
	
	def options_clicked(self,button):
		
		self.options_popover.show_all()
		
	#def options_clicked
	
	
	def import_clicked(self,button):
		
		print("IMPORT")
		self.options_popover.hide()
		
	#def import_clicked
	
	
	def export_clicked(self,button):
		
		print("EXPORT")
		self.options_popover.hide()
		
	#def import_clicked
	
	# ############################################
	
	
if __name__=="__main__":
	
	pmw=PanelDesignWindow()
	
